/*%%  (c) Copyright 1993, 1994 Hewlett-Packard Company			 */
/*%%  (c) Copyright 1993, 1994 International Business Machines Corp.	 */
/*%%  (c) Copyright 1993, 1994 Sun Microsystems, Inc.			 */
/*%%  (c) Copyright 1993, 1994 Novell, Inc.					 */
/*%%  $Revision: $							 */
/*
 * Copyright (c) 1988 by Sun Microsystems, Inc.
 */

#pragma ident   "@(#)iserase.c 1.7     98/09/22 SMI; TSOL 2.x"

/*
 * iserase.c
 *
 * Description:
 *	Erase an ISAM file.
 */


#include "isam_impl.h"
#include <sys/time.h>

static void _unlink_datfile(char *);
static void _unlink_indfile(char *);
static void _unlink_varfile(char *);
static int _amerase(Bytearray *, struct errcode *);

/*
 * isfd = iserase(isfname, mode)
 *
 *
 * Errors:
 *	EBADFILE ISAM file is corrupted or it is not an NetISAM file
 *	EFLOCKED The file is exclusively locked by other process.
 *	EFNAME	Invalid ISAM file name
 *	EFNAME	ISAM file does not exist
 *	ETOOMANY Too many ISAM file descriptors are in use (128 is the limit)
 *
 * The following error code is "borrowed" from UNIX:
 *	EACCES	UNIX file system protection denies access to the file:
 *			- mode is INOUT or OUTPUT and ISAM file is on
 *			  a Read-Only mounted file system
 *		 	- UNIX file permissions don't allow access to the file
 */

int
iserase(isfname)
	char		*isfname;
{
	Isfd		isfd;
	Fab			*fab;

	/*
	 * Open the file
	 */
	if ((isfd = isopen(isfname, ISINOUT)) == -1)
	return (ISERROR);			 /* iserrno is set */

	/*
	 * Get File Access Block.
	 */
	if ((fab = _isfd_find(isfd)) == NULL) {
	_isfatal_error("iserase() cannot find FAB", (char *) 0);
	_setiserrno2(EFATAL, '9', '0');
	return ISERROR;
	}

	if (_amerase(&fab->isfhandle, &fab->errcode)
			&& fab->errcode.iserrno != ENOENT) {
	_seterr_errcode(&fab->errcode);
	(void)isclose(isfd);
	return (ISERROR);
	}

	_fab_destroy(fab);				 /* Deallocate Fab object */
	_isfd_delete(isfd);

	return (ISOK);				 /* Successful iserase() */
}

/*
 * _amerase(isfhandle)
 *
 * _amerase() erases ISAM file
 *
 * Input params:
 *	isfhandle	Handle of ISAM file
 *
 * Output params:
 *	errcode		Error code
 *
 */

static int
_amerase(isfhandle, errcode)
	Bytearray		*isfhandle;
	struct errcode	*errcode;
{
	Fcb			*fcb;
	char		*isfname = _getisfname(isfhandle);

	_isam_entryhook();

	/*
	 * Get FCB corresponding to the isfhandle handle.
	 */
	if ((fcb = _openfcb(isfhandle, errcode)) == NULL) {
	goto ERROR;
	}

	/*
	 * Delete FCB and remove it from FCB cache.
	 */
	(void) _watchfd_decr(_isfcb_nfds(fcb));
	_isfcb_close(fcb);
	_mngfcb_delete(isfhandle);

	/*
	 * Unlink all UNIX files.
	 */
	_unlink_datfile(isfname);
	_unlink_indfile(isfname);
	_unlink_varfile(isfname);

	_isam_exithook();
	return (ISOK);

 ERROR:

	_isam_exithook();
	return (ISERROR);
}


static void
_unlink_datfile(isfname)
	char	*isfname;
{
	_make_isfname_with_remove(DAT_FILE, isfname);
}


static void
_unlink_indfile(isfname)
	char	*isfname;
{
	_make_isfname_with_remove(IND_FILE, isfname);
}


static void
_unlink_varfile(isfname)
	char	*isfname;
{
	_make_isfname_with_remove(VAR_FILE, isfname);
}
