/*%%	(c) Copyright 1993, 1994 Hewlett-Packard Company					*/
/*%%	(c) Copyright 1993, 1994 International Business Machines Corp.	*/
/*%%	(c) Copyright 1993, 1994 Sun Microsystems, Inc.						*/
/*%%	(c) Copyright 1993, 1994 Novell, Inc.									*/

/*
 * Copyright (c) 1988 by Sun Microsystems, Inc.
*/

#pragma ident   "@(#)isfname.c 1.8     98/09/22 SMI; TSOL 2.x"

/*
 * isfname.c  1.10, last updated (yy/mm/dd) 98/08/12
 *
 * Description:
 *	Functions to translate ISAM file name to .rec, .ind, and .var file
 *	names. Also, several auxiliary functions that deal with path names.
 *
*/

#include <stdlib.h>
#include "isam_impl.h"



/*
 * _make_isfname_with_open(file_type, isfname, flags, perms)
 *
 * Add the appropriate suffix onto the path name provided, and open
 * the resulting path name..
*/

int
_make_isfname_with_open(u_char file_type, char* isfname, int flags, mode_t perms)
{
	char *	filename;
	int		fd=-1;

	if (filename = _make_isfname(file_type, isfname))
	 {
		fd = open(filename, flags, perms);
		free(filename);
	 }
	return fd;
}

/*
 * _make_isfname_with_remove(file_type, isfname)
 *
 * Add the appropriate suffix onto the path name provided, and attempt
 * to delete the resulting path name.
*/

void
_make_isfname_with_remove(u_char file_type, char* isfname)
{
	char *	filename;

	if (filename = _make_isfname(file_type, isfname))
	 {
		(void)unlink(filename);
		free(filename);
	 }
}

/*
 * _make_isfname(file_type, isfname)
 *
 * Return path to file corresponding to file_type, with path isfname.
 * Conversion is done in place.
*/

char*
_make_isfname(u_char file_type, char* isfname)
{
	char *	cp;
	char *	filename;

	switch(file_type)
	 {
		case VAR_FILE:
			cp = VAR_SUFFIX;
			break;

		case IND_FILE:
			cp = IND_SUFFIX;
			break;

		case DAT_FILE:
			cp = DAT_SUFFIX;
			break;

		default:
			cp = "";
			break;
	 }

	filename = (char *)malloc((strlen(cp) + strlen(isfname)) + 1);
	if (filename)
	 {
		strcat(strcpy(filename, isfname), cp);
	 }
	return filename;
}

/*
 * _removelast(path)
 *
 * Remove last element of path. E.g. /usr/db/part yields /usr/db.
 */

void
_removelast(path)
	 char		*path;
{
	register char	*p;

	for (p = path + strlen(path); *--p != '/' && p >= path; )
	*p = '\0';
}

/*
 * _lastelement(path)
 *
 * Return pointer to the last element in the path.
 * E.g.: _lastelement("/usr/temp") returns "temp".
 */

char *
_lastelement(path)
	 char	*path;
{
	register	char	*p;

	p = path + strlen(path);

	while (*--p != '/' && p > path)
	{ ; }

	return ((*p == '/') ? (p + 1) : p);
}
