/*%%  (c) Copyright 1993, 1994 Hewlett-Packard Company			 */
/*%%  (c) Copyright 1993, 1994 International Business Machines Corp.	 */
/*%%  (c) Copyright 1993, 1994, 1998 by Sun Microsystems, Inc.			 */
/*%%  (c) Copyright 1993, 1994 Novell, Inc.				 */


#pragma ident	"@(#)isrename.c	1.8	98/09/22 SMI; TSOL 2.x"

/*
 * Copyright (c) 1988 by Sun Microsystems, Inc.
 */

/*
 * isrename.c  1.11, last updated (yy/mm/dd) 98/08/12
 *
 * Description:
 *	Rename an ISAM file.
 */


#include <stdlib.h>
#include "isam_impl.h"
#include <sys/time.h>

static void _removelast2(char *);

static void _rename_file(u_char, char *, char *);

static int _amrename(Bytearray *, char *, struct errcode *);


/*
 * isfd = isrename(oldname, newname)
 *
 *
 * Errors:
 *	EBADFILE ISAM file is corrupted or it is not an NetISAM file
 *	EFLOCKED The file is exclusively locked by other process.
 *	EFNAME	Invalid ISAM file name
 *	EFNAME	ISAM file does not exist
 *	ETOOMANY Too many ISAM file descriptors are in use (128 is the limit)
 *
 * The following error code is "borrowed" from UNIX:
 *	EACCES	UNIX file system protection denies access to the file:
 *		 - mode is INOUT or OUTPUT and ISAM file is on
 *		   a Read-Only mounted file system
 *		 - UNIX file permissions don't allow access to the file
 */

int
isrename(oldname, newname)
    char		*oldname, *newname;
{
    Isfd		isfd, isfd2;
    Fab			*fab;
    char		olddir [MAXPATHLEN+1];
    char		newdir [MAXPATHLEN+1];

    /*
     * Check that the odl and new filename are in the same directory.
     */
    (void)strncpy(olddir, oldname, MAXPATHLEN);
    olddir[MAXPATHLEN] = 0;
    _removelast2(olddir);
    (void)strncpy(newdir, newname, MAXPATHLEN);
    newdir[MAXPATHLEN] = 0;
    _removelast2(newdir);

    if (strcmp(newdir, olddir) != 0) {
	_setiserrno2(EFNAME, 9, 0);
	return (ISERROR);
    }

    /*
     * Open the file
     */
    if ((isfd = isopen(oldname, ISINOUT)) == -1)
	return (ISERROR);		     /* iserrno is set */

    /*
     * Reject rename if 'newfile' exists.
     */
    if ((isfd2 = isopen(newname, ISINOUT)) != -1 || iserrno != ENOENT) {

	_setiserrno2(EEXIST, 9, 0);
	if (isfd2 >= 0) (void)isclose(isfd2);
	return (ISERROR);		     /* iserrno is set */
    }

    /*
     * Get File Access Block.
     */
    if ((fab = _isfd_find(isfd)) == NULL) {
	_isfatal_error("isrename() cannot find FAB", (char *) 0);
	_setiserrno2(EFATAL,'9','0');
	return (ISERROR);
    }

    if (_amrename(&fab->isfhandle, _lastelement(newname), &fab->errcode)) {
	_seterr_errcode(&fab->errcode);
	(void)isclose(isfd);
	return (ISERROR);
    }

    _fab_destroy(fab);			     /* Deallocate Fab object */
    _isfd_delete(isfd);

    /*
     * We must unlink() the .rec file, or isbuild() with the same
     * following immediatly would fail because for NFS files the client
     * still thinks that the file exists for a few seconds.
     */

    _make_isfname_with_remove(DAT_FILE, oldname);

    return (ISOK);			     /* Successful isrename() */
}

/*
 * _removelast2(path)
 *
 * Remove last element of path. E.g. /usr/db/part yields /usr/db.
 * Unlike _removelast() that path does not have to start with '/'.
 */

static void
_removelast2(path)
    char		*path;
{
    register char	*p;

    for (p = path + strlen(path); *--p != '/' && p >= path; )
	*p = '\0';
}

/*
 * _amrename(isfhandle, newname)
 *
 * _amrename() renames ISAM file
 *
 * Input params:
 *	isfhandle	Handle of ISAM file
 *
 * Output params:
 *	errcode		Error code
 *
 */


static int
_amrename(isfhandle, newname, errcode)
    Bytearray		*isfhandle;
    char		*newname;
    struct errcode	*errcode;
{
    Fcb			*fcb;
    char		*isfname = _getisfname(isfhandle);

    _isam_entryhook();

    /*
     * Get FCB corresponding to the isfhandle handle.
     */
    if ((fcb = _openfcb(isfhandle, errcode)) == NULL) {
	goto ERROR;
    }

    /*
     * Delete FCB and remove it from FCB cache.
     */
    (void) _watchfd_decr(_isfcb_nfds(fcb));
    _isfcb_close(fcb);
    _mngfcb_delete(isfhandle);

    /*
     * Rename all UNIX files.
     */
    _rename_file(DAT_FILE, isfname, newname);
    _rename_file(IND_FILE, isfname, newname);
    _rename_file(VAR_FILE, isfname, newname);

    _isam_exithook();
    return (ISOK);

 ERROR:

    _isam_exithook();
    return (ISERROR);
}


static void
_rename_file(u_char file_type, char* isfname, char* newname)
{
    char *	cp,
	 *	newfile,
	 *	oldfile;

    cp = (char *)malloc(strlen(isfname) + strlen(newname) + 1);
    if (!cp) {
	return;
    }
    strcpy(cp, isfname);

    /*
     * Replace the last element of the old path with newname.
     */
    _removelast(cp);
    if (strcmp(cp, "/") != 0)
	(void) strcat(cp, "/");
    strcat(cp, newname);

    if (!(oldfile = _make_isfname(file_type, isfname))) {
	free(cp);
	return;
    }
    if (!(newfile = _make_isfname(file_type, cp))) {
	free(oldfile);
	free(cp);
	return;
    }

    (void)rename(oldfile, newfile);
    free(newfile);
    free(oldfile);
    free(cp);
}
